@@ -19,15 +19,15 @@ from glue.icons.qt import get_icon
 from glue.utils.qt import get_qapp
 from glue.app.qt.actions import action
 from glue.dialogs.data_wizard.qt import data_wizard
+from glue.dialogs.link_editor.qt import LinkEditor
 from glue.app.qt.edit_subset_mode_toolbar import EditSubsetModeToolBar
 from glue.app.qt.mdi_area import GlueMdiArea, GlueMdiSubWindow
 from glue.app.qt.layer_tree_widget import PlotAction, LayerTreeWidget
 from glue.app.qt.preferences import PreferencesDialog
-from glue.viewers.common.qt.mpl_widget import defer_draw
 from glue.viewers.common.qt.data_viewer import DataViewer
-from glue.viewers.image.qt import ImageWidget
-from glue.viewers.scatter.qt import ScatterWidget
-from glue.utils import nonpartial
+from glue.viewers.scatter.qt import ScatterViewer
+from glue.viewers.image.qt import ImageViewer
+from glue.utils import nonpartial, defer_draw
 from glue.utils.qt import (pick_class, GlueTabBar,
                            set_cursor_cm, messagebox_on_error, load_ui)
 
@@ -74,32 +74,10 @@ def _fix_ipython_pylab():
     except UsageError:
         pass
 
-
-def status_pixmap(attention=False):
-    """
-    A small icon to grab attention
-
-    :param attention: If True, return attention-grabbing pixmap
-    """
-    color = Qt.red if attention else Qt.lightGray
-
-    pm = QtGui.QPixmap(15, 15)
-    p = QtGui.QPainter(pm)
-    b = QtGui.QBrush(color)
-    p.fillRect(-1, -1, 20, 20, b)
-    return pm
-
-
-class ClickableLabel(QtWidgets.QLabel):
-
-    """
-    A QtWidgets.QLabel you can click on to generate events
-    """
-
-    clicked = QtCore.Signal()
-
-    def mousePressEvent(self, event):
-        self.clicked.emit()
+    # Make sure we disable interactive mode (where figures get redrawn for
+    # every single Matplotlib command)
+    import matplotlib
+    matplotlib.interactive(False)
 
 
 class GlueLogger(QtWidgets.QWidget):
@@ -108,8 +86,15 @@ class GlueLogger(QtWidgets.QWidget):
     A window to display error messages
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, button_console, parent=None):
+
         super(GlueLogger, self).__init__(parent)
+
+        self.button_console = button_console
+        self.button_stylesheet = button_console.styleSheet()
+
+        self.button_console.clicked.connect(self._show)
+
         self._text = QtWidgets.QTextEdit()
         self._text.setTextInteractionFlags(Qt.TextSelectableByMouse)
 
@@ -119,14 +104,14 @@ class GlueLogger(QtWidgets.QWidget):
         report = QtWidgets.QPushButton("Send Bug Report")
         report.clicked.connect(nonpartial(self._send_report))
 
-        self.stderr = sys.stderr
-        sys.stderr = self
+        if isinstance(sys.stderr, GlueLogger):
+            if isinstance(sys.stderr._stderr_original, GlueLogger):
+                raise Exception('Too many nested GlueLoggers')
+            self._stderr_original = sys.stderr._stderr_original
+        else:
+            self._stderr_original = sys.stderr
 
-        self._status = ClickableLabel()
-        self._status.setToolTip("View Errors and Warnings")
-        self._status.clicked.connect(self._show)
-        self._status.setPixmap(status_pixmap())
-        self._status.setContentsMargins(0, 0, 0, 0)
+        sys.stderr = self
 
         l = QtWidgets.QVBoxLayout()
         h = QtWidgets.QHBoxLayout()
@@ -142,21 +127,20 @@ class GlueLogger(QtWidgets.QWidget):
 
         self.setLayout(l)
 
-    @property
-    def status_light(self):
-        """
-        The icon representing the status of the log
-        """
-        return self._status
+    def _set_console_button(self, attention):
+        if attention:
+            self.button_console.setStyleSheet('color: red; text-decoration: underline;')
+        else:
+            self.button_console.setStyleSheet(self.button_stylesheet)
 
     def write(self, message):
         """
         Interface for sys.excepthook
         """
-        self.stderr.write(message)
+        self._stderr_original.write(message)
         self._text.moveCursor(QtGui.QTextCursor.End)
         self._text.insertPlainText(message)
-        self._status.setPixmap(status_pixmap(attention=True))
+        self._set_console_button(attention=True)
 
     def flush(self):
         """
@@ -176,7 +160,7 @@ class GlueLogger(QtWidgets.QWidget):
         Erase the log
         """
         self._text.setText('')
-        self._status.setPixmap(status_pixmap(attention=False))
+        self._set_console_button(attention=False)
         self.close()
 
     def _show(self):
@@ -193,12 +177,16 @@ class GlueLogger(QtWidgets.QWidget):
         if event.key() == Qt.Key_Escape:
             self.hide()
 
+    def closeEvent(self, event):
+        if sys.stderr is self:
+            sys.stderr = self._stderr_original
+
 
 class GlueApplication(Application, QtWidgets.QMainWindow):
 
     """ The main GUI application for the Qt frontend"""
 
-    def __init__(self, data_collection=None, session=None, maximized=True):
+    def __init__(self, data_collection=None, session=None):
 
         self.app = get_qapp()
 
@@ -233,7 +221,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         lwidget.ui.layerTree.addAction(a)
         lwidget.bind_selection_to_edit_subset()
 
-        self._tweak_geometry(maximized=maximized)
+        self._tweak_geometry()
         self._create_actions()
         self._create_menu()
         self._connect()
@@ -254,20 +242,112 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         self._ui.data_layers.setLayout(self._vb)
         self._layer_widget = lw
 
-        # log window + status light
-        self._log = GlueLogger()
+        # Data toolbar
+
+        self._data_toolbar = QtWidgets.QToolBar()
+
+        self._data_toolbar.setIconSize(QtCore.QSize(16, 16))
+
+        self._button_open_data = QtWidgets.QToolButton()
+        self._button_open_data.setText("Open Data")
+        self._button_open_data.setIcon(get_icon('glue_open'))
+        self._button_open_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_open_data.clicked.connect(nonpartial(self._choose_load_data))
+
+        self._data_toolbar.addWidget(self._button_open_data)
+
+        self._button_link_data = QtWidgets.QToolButton()
+        self._button_link_data.setText("Link Data")
+        self._button_link_data.setIcon(get_icon('glue_link'))
+        self._button_link_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_link_data.clicked.connect(self._set_up_links)
+
+        self._data_toolbar.addWidget(self._button_link_data)
+
+        self._button_ipython = QtWidgets.QToolButton()
+        self._button_ipython.setCheckable(True)
+        self._button_ipython.setText("IPython Terminal")
+        self._button_ipython.setIcon(get_icon('IPythonConsole'))
+        self._button_ipython.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_ipython.clicked.connect(self._toggle_terminal)
+
+        self._data_toolbar.addWidget(self._button_ipython)
+
+        self._button_open_data = QtWidgets.QToolButton()
+        self._button_open_data.setText("Open Session")
+        self._button_open_data.setIcon(get_icon('glue_open'))
+        self._button_open_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_open_data.clicked.connect(nonpartial(self._restore_session))
+
+        self._data_toolbar.addWidget(self._button_open_data)
+
+        self._button_open_data = QtWidgets.QToolButton()
+        self._button_open_data.setText("Save Session")
+        self._button_open_data.setIcon(get_icon('glue_filesave'))
+        self._button_open_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_open_data.clicked.connect(nonpartial(self._choose_save_session))
+
+        self._data_toolbar.addWidget(self._button_open_data)
+
+        spacer = QtWidgets.QWidget()
+        spacer.setMinimumSize(20, 10)
+        spacer.setSizePolicy(QtWidgets.QSizePolicy.Fixed,
+                             QtWidgets.QSizePolicy.Preferred)
+
+        self._data_toolbar.addWidget(spacer)
+
+        self.addToolBar(self._data_toolbar)
+
+        # Selection mode toolbar
+
+        tbar = EditSubsetModeToolBar()
+        self._mode_toolbar = tbar
+
+        self.addToolBar(self._mode_toolbar)
+
+        # Error console toolbar
+
+        self._console_toolbar = QtWidgets.QToolBar()
+
+        self._console_toolbar.setIconSize(QtCore.QSize(16, 16))
+
+        spacer = QtWidgets.QWidget()
+        spacer.setSizePolicy(QtWidgets.QSizePolicy.Expanding,
+                             QtWidgets.QSizePolicy.Preferred)
+
+        self._console_toolbar.addWidget(spacer)
+
+        self._button_preferences = QtWidgets.QToolButton()
+        self._button_preferences.setText("Preferences")
+        self._button_preferences.setIcon(get_icon('glue_settings'))
+        self._button_preferences.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_preferences.clicked.connect(nonpartial(self._edit_settings))
+
+        self._console_toolbar.addWidget(self._button_preferences)
+
+        self._button_console = QtWidgets.QToolButton()
+        self._button_console.setText("View Error Console")
+        self._button_console.setToolButtonStyle(Qt.ToolButtonTextOnly)
+
+        self._console_toolbar.addWidget(self._button_console)
+
+        self.addToolBar(self._console_toolbar)
+
+        self._log = GlueLogger(button_console=self._button_console)
         self._log.window().setWindowTitle("Console Log")
         self._log.resize(550, 550)
-        self.statusBar().addPermanentWidget(self._log.status_light)
-        self.statusBar().setContentsMargins(2, 0, 20, 2)
-        self.statusBar().setSizeGripEnabled(False)
+        self._log.hide()
 
-    def _tweak_geometry(self, maximized=True):
+    def _set_up_links(self, event):
+        LinkEditor.update_links(self.data_collection)
+
+    def _tweak_geometry(self):
         """Maximize window by default."""
-        if maximized:
-            self.setWindowState(Qt.WindowMaximized)
-        self._ui.main_splitter.setSizes([100, 800])
-        self._ui.data_plot_splitter.setSizes([100, 150, 250])
+        self._ui.main_splitter.setStretchFactor(0, 0.1)
+        self._ui.main_splitter.setStretchFactor(1, 0.9)
+        self._ui.data_plot_splitter.setStretchFactor(0, 0.25)
+        self._ui.data_plot_splitter.setStretchFactor(1, 0.5)
+        self._ui.data_plot_splitter.setStretchFactor(2, 0.25)
 
     @property
     def tab_widget(self):
@@ -401,7 +481,9 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
                 widget.setLayout(layout)
             while layout.count():
                 layout.takeAt(0).widget().hide()
-            widget.setTitle(title)
+
+        self._ui.plot_options_label.setText("Plot Options")
+        self._ui.plot_layers_label.setText("Plot Layers")
 
     def _update_plot_dashboard(self, sub_window):
         self._clear_dashboard()
@@ -422,11 +504,11 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         options_widget.show()
 
         if title:
-            self._ui.plot_options.setTitle("Plot Options - %s" % title)
-            self._ui.plot_layers.setTitle("Plot Layers - %s" % title)
+            self._ui.plot_options_label.setText("Plot Options - %s" % title)
+            self._ui.plot_layers_label.setText("Plot Layers - %s" % title)
         else:
-            self._ui.plot_options.setTitle("Plot Options")
-            self._ui.plot_layers.setTitle("Plot Layers")
+            self._ui.plot_options_label.setText("Plot Options")
+            self._ui.plot_layers_label.setText("Plot Layers")
 
         self._update_focus_decoration()
 
@@ -497,24 +579,6 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         mbar.addMenu(menu)
 
-        menu = QtWidgets.QMenu(mbar)
-        menu.setTitle("&Toolbars")
-        tbar = EditSubsetModeToolBar()
-        self._mode_toolbar = tbar
-        self.addToolBar(tbar)
-        tbar.hide()
-        a = QtWidgets.QAction("Selection Mode &Toolbar", menu)
-        a.setCheckable(True)
-        a.toggled.connect(tbar.setVisible)
-        try:
-            tbar.visibilityChanged.connect(a.setChecked)
-        except AttributeError:  # Qt < 4.7. QtCore.Signal not supported
-            pass
-
-        menu.addAction(a)
-        menu.addActions(tbar.actions())
-        mbar.addMenu(menu)
-
         menu = QtWidgets.QMenu(mbar)
         menu.setTitle("&Plugins")
         menu.addAction(self._actions['plugin_manager'])
@@ -675,10 +739,10 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         from glue.config import qt_client
 
-        if data and data.ndim == 1 and ScatterWidget in qt_client.members:
-            default = ScatterWidget
-        elif data and data.ndim > 1 and ImageWidget in qt_client.members:
-            default = ImageWidget
+        if data and data.ndim == 1 and ScatterViewer in qt_client.members:
+            default = ScatterViewer
+        elif data and data.ndim > 1 and ImageViewer in qt_client.members:
+            default = ImageViewer
         else:
             default = None
 
@@ -746,12 +810,12 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
             self.close()
         return ga
 
-    def _reset_session(self, show=True):
+    def _reset_session(self, show=True, warn=True):
         """
         Reset session to clean state.
         """
 
-        if not os.environ.get('GLUE_TESTING'):
+        if not os.environ.get('GLUE_TESTING') and warn:
             buttons = QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel
             dialog = QtWidgets.QMessageBox.warning(
                 self, "Confirm Close",
@@ -761,8 +825,15 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
             if not dialog == QtWidgets.QMessageBox.Ok:
                 return
 
+        # Make sure the closeEvent gets executed to close the GlueLogger
+        self._log.close()
+        self.app.processEvents()
+
         ga = GlueApplication()
-        ga.show()
+        ga.start(block=False)
+
+        # We need to close this after we open the next application otherwise
+        # Qt will quit since there are no actively open windows.
         self.close()
 
         return ga
@@ -786,14 +857,15 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         """
         ga = Application.restore_session(path)
         if show:
-            ga.show()
+            ga.start(block=False)
         return ga
 
-    def has_terminal(self):
+    def has_terminal(self, create_if_not=True):
         """
         Returns True if the IPython terminal is present.
         """
-        self._create_terminal()  # ensure terminal is setup
+        if self._terminal is None and create_if_not:
+            self._create_terminal()
         return self._terminal is not None
 
     def _create_terminal(self):
@@ -801,17 +873,6 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         if self._terminal is not None:  # already set up
             return
 
-        if hasattr(self, '_terminal_exception'):  # already failed to set up
-            return
-
-        self._terminal_button = QtWidgets.QToolButton(self._ui)
-        self._terminal_button.setToolTip("Toggle IPython Prompt")
-        i = get_icon('IPythonConsole')
-        self._terminal_button.setIcon(i)
-        self._terminal_button.setIconSize(QtCore.QSize(25, 25))
-
-        self._layer_widget.ui.button_row.addWidget(self._terminal_button)
-
         try:
             widget = glue_terminal(data_collection=self._data,
                                    dc=self._data,
@@ -819,30 +880,12 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
                                    session=self.session,
                                    application=self,
                                    **vars(env))
-        except IPythonTerminalError as exc:
-            self._terminal = None
-            self._terminal_button.setEnabled(False)
-            self._terminal_button.setToolTip(str(exc))
-            self._terminal_exception = str(exc)
-            return
-
-        self._terminal_button.clicked.connect(self._toggle_terminal)
-        self._terminal = self.add_widget(widget)
-        self._hide_terminal()
-
-    def _setup_terminal_error_dialog(self, exception):
-        """ Reassign the terminal toggle button to show dialog on error"""
-        title = "Terminal unavailable"
-        msg = ("Glue encountered an error trying to start the Terminal"
-               "\nReason:\n%s" % exception)
-
-        def show_msg():
-            mb = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Critical,
-                             title, msg)
-            mb.setDetailedText(self._terminal_exception)
-            mb.exec_()
-
-        self._terminal_button.clicked.connect(show_msg)
+        except IPythonTerminalError:
+            self._button_ipython.setEnabled(False)
+        else:
+            self._terminal = self.add_widget(widget)
+            self._terminal.closed.connect(self._on_terminal_close)
+            self._hide_terminal()
 
     def _toggle_terminal(self):
         if self._terminal.isVisible():
@@ -856,6 +899,12 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
                 warnings.warn("An unexpected error occurred while "
                               "trying to show the terminal")
 
+    def _on_terminal_close(self):
+        if self._button_ipython.isChecked():
+            self._button_ipython.blockSignals(True)
+            self._button_ipython.setChecked(False)
+            self._button_ipython.blockSignals(False)
+
     def _hide_terminal(self):
         self._terminal.hide()
 
@@ -863,7 +912,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         self._terminal.show()
         self._terminal.widget().show()
 
-    def start(self, size=None, position=None):
+    def start(self, size=None, position=None, block=True, maximized=True):
         """
         Show the GUI and start the application.
 
@@ -876,7 +925,10 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
             The default position of the application
         """
         self._create_terminal()
-        self.show()
+        if maximized:
+            self.showMaximized()
+        else:
+            self.show()
         if size is not None:
             self.resize(*size)
         if position is not None:
@@ -890,20 +942,11 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         # XXX find out a better place for this
         _fix_ipython_pylab()
 
-        return self.app.exec_()
+        if block:
+            return self.app.exec_()
 
     exec_ = start
 
-    def keyPressEvent(self, event):
-        """Hold down modifier keys to temporarily set edit mode"""
-        mod = event.modifiers()
-        if mod == Qt.ShiftModifier:
-            self._mode_toolbar.set_mode('or')
-
-    def keyReleaseEvent(self, event):
-        """Unset any temporary edit mode"""
-        self._mode_toolbar.unset_mode()
-
     def dragEnterEvent(self, event):
         if event.mimeData().hasUrls():
             event.accept()
@@ -930,6 +973,10 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
     def closeEvent(self, event):
         """Emit a message to hub before closing."""
+        for tab in self.viewers:
+            for viewer in tab:
+                viewer.close(warn=False)
+        self._log.close()
         self._hub.broadcast(ApplicationClosedMessage(None))
         event.accept()
 
