@@ -10,7 +10,7 @@ import os
 from qtpy import QtCore, QtWidgets, QtGui
 from qtpy.QtCore import Qt
 
-from glue.core.edit_subset_mode import AndMode, OrMode, XorMode, AndNotMode
+from glue.core.edit_subset_mode import AndMode, OrMode, XorMode, AndNotMode, EditSubsetMode
 from glue.config import single_subset_action
 from glue import core
 from glue.dialogs.link_editor.qt import LinkEditor
@@ -40,7 +40,7 @@ class LayerAction(QtWidgets.QAction):
 
     def __init__(self, layer_tree_widget):
         self._parent = layer_tree_widget.ui.layerTree
-        super(LayerAction, self).__init__(self._title.title(), self._parent)
+        super(LayerAction, self).__init__(self._title.capitalize(), self._parent)
         self._layer_tree = layer_tree_widget
         if self._icon:
             self.setIcon(get_icon(self._icon))
@@ -219,7 +219,7 @@ class MaskifySubsetAction(LayerAction):
 
 class ExportDataAction(LayerAction):
 
-    _title = "Export data"
+    _title = "Export data values"
     _tooltip = "Save the data to a file"
 
     def _can_trigger(self):
@@ -231,15 +231,47 @@ class ExportDataAction(LayerAction):
         from glue.core.data_exporters.qt.dialog import export_data
         export_data(data)
 
+
 class ExportSubsetAction(ExportDataAction):
 
-    _title = "Export subset"
+    _title = "Export subset values"
     _tooltip = "Save the data subset to a file"
 
     def _can_trigger(self):
         return self.single_selection_subset()
 
 
+class ImportSubsetMaskAction(LayerAction):
+
+    _title = "Import subset mask(s)"
+    _tooltip = "Import subset mask from a file"
+
+    def _can_trigger(self):
+        return self.single_selection_subset() or self.single_selection_data()
+
+    def _do_action(self):
+        assert self._can_trigger()
+        data = self.selected_layers()[0]
+        from glue.io.qt.subset_mask import QtSubsetMaskImporter
+        QtSubsetMaskImporter().run(data, self._layer_tree._data_collection)
+
+
+class ExportSubsetMaskAction(LayerAction):
+
+    _title = "Export subset mask(s)"
+    _tooltip = "Export subset mask to a file"
+
+    def _can_trigger(self):
+        return (len(self.data_collection.subset_groups) > 0 and
+                (self.single_selection_subset() or self.single_selection_data()))
+
+    def _do_action(self):
+        assert self._can_trigger()
+        data = self.selected_layers()[0]
+        from glue.io.qt.subset_mask import QtSubsetMaskExporter
+        QtSubsetMaskExporter().run(data)
+
+
 class CopyAction(LayerAction):
     _title = "Copy subset"
     _tooltip = "Copy the definition for the selected subset"
@@ -401,8 +433,6 @@ class LayerTreeWidget(QtWidgets.QMainWindow):
 
         self.ui = load_ui('layer_tree_widget.ui', None, directory=os.path.dirname(__file__))
         self.setCentralWidget(self.ui)
-        self.ui.layerAddButton.setIcon(get_icon('glue_open'))
-        self.ui.layerRemoveButton.setIcon(get_icon('glue_delete'))
 
         self._signals = LayerCommunicator()
         self._is_checkable = True
@@ -412,10 +442,10 @@ class LayerTreeWidget(QtWidgets.QMainWindow):
         self._actions = {}
 
         self._create_actions()
-        self._connect()
         self._data_collection = None
         self._hub = None
         self.ui.layerTree.setDragEnabled(True)
+        self.setMinimumSize(300, 0)
 
     @property
     def data_collection(self):
@@ -452,33 +482,14 @@ class LayerTreeWidget(QtWidgets.QMainWindow):
         """ Return the list of actions attached to this widget """
         return self.ui.layerTree.actions()
 
-    def _connect(self):
-        """ Connect widget signals to methods """
-        self._actions['link'] = LinkAction(self)
-        self.ui.layerAddButton.clicked.connect(nonpartial(self._load_data))
-        self.ui.layerRemoveButton.clicked.connect(self._actions['delete'].trigger)
-        self.ui.linkButton.set_action(self._actions['link'])
-        self.ui.newSubsetButton.set_action(self._actions['new'], text=False)
-
-        rbut = self.ui.layerRemoveButton
-
-        def update_enabled():
-            return rbut.setEnabled(self._actions['delete'].isEnabled())
-        self.ui.layerTree.selection_changed.connect(update_enabled)
-
     def bind_selection_to_edit_subset(self):
         self.ui.layerTree.selection_changed.connect(
             self._update_editable_subset)
 
     def _update_editable_subset(self):
         """Update edit subsets to match current selection"""
-        layers = self.selected_layers()
-        layers.extend(s for l in layers
-                      if isinstance(l, core.SubsetGroup)
-                      for s in l.subsets)
-
-        for data in self.data_collection:
-            data.edit_subset = [s for s in data.subsets if s in layers]
+        mode = EditSubsetMode()
+        mode.edit_subset = [s for s in self.selected_layers() if isinstance(s, core.SubsetGroup)]
 
     def _create_component(self):
         dialog = CustomComponentWidget(self.data_collection)
@@ -491,8 +502,12 @@ class LayerTreeWidget(QtWidgets.QMainWindow):
         sep.setSeparator(True)
         tree.addAction(sep)
 
+        # Actions relating to I/O
         self._actions['save_data'] = ExportDataAction(self)
         self._actions['save_subset'] = ExportSubsetAction(self)
+        self._actions['import_subset_mask'] = ImportSubsetMaskAction(self)
+        self._actions['export_subset_mask'] = ExportSubsetMaskAction(self)
+
         self._actions['copy'] = CopyAction(self)
         self._actions['paste'] = PasteAction(self)
         self._actions['paste_special'] = PasteSpecialAction(self)
@@ -503,11 +518,12 @@ class LayerTreeWidget(QtWidgets.QMainWindow):
         self._actions['facet'] = FacetAction(self)
         self._actions['merge'] = MergeAction(self)
         self._actions['maskify'] = MaskifySubsetAction(self)
+        self._actions['link'] = LinkAction(self)
 
         # new component definer
-        sep = QtWidgets.QAction("", tree)
-        sep.setSeparator(True)
-        tree.addAction(sep)
+        separator = QtWidgets.QAction("sep", tree)
+        separator.setSeparator(True)
+        tree.addAction(separator)
 
         a = action("Define new component", self,
                  tip="Define a new component using python expressions")
