@@ -448,6 +448,22 @@ class DataExporterRegistry(Registry):
             yield member
 
 
+class SubsetMaskExporterRegistry(DataExporterRegistry):
+    """
+    Stores mask exporters. Mask exporters should take a filename followed by
+    a dictionary of Numpy boolean arrays all with the same dimensions.
+    """
+    item = namedtuple('SubsetMaskExporter', 'function label extension')
+
+
+class SubsetMaskImporterRegistry(DataExporterRegistry):
+    """
+    Stores mask importers. Mask importers should take a filename and return a
+    dictionary of Numpy boolean arrays.
+    """
+    item = namedtuple('SubsetMaskImporter', 'function label extension')
+
+
 class QtClientRegistry(Registry):
     """
     Stores QT widgets to visualize data.
@@ -585,8 +601,6 @@ class BooleanSetting(object):
 
 qt_client = QtClientRegistry()
 viewer_tool = ViewerToolRegistry()
-data_factory = DataFactoryRegistry()
-data_exporter = DataExporterRegistry()
 link_function = LinkFunctionRegistry()
 link_helper = LinkHelperRegistry()
 colormaps = ColormapRegistry()
@@ -603,6 +617,13 @@ qglue_parser = QGlueParserRegistry()
 auto_refresh = BooleanSetting(False)
 enable_contracts = BooleanSetting(False)
 
+# Data and subset I/O
+data_factory = DataFactoryRegistry()
+data_exporter = DataExporterRegistry()
+subset_mask_exporter = SubsetMaskExporterRegistry()
+subset_mask_importer = SubsetMaskImporterRegistry()
+
+
 
 def load_configuration(search_path=None):
     ''' Find and import a config.py file
@@ -674,3 +695,4 @@ settings.add('DATA_ALPHA', 0.8, validator=float)
 settings.add('BACKGROUND_COLOR', '#FFFFFF')
 settings.add('FOREGROUND_COLOR', '#000000')
 settings.add('SHOW_LARGE_DATA_WARNING', True, validator=bool)
+settings.add('INDIVIDUAL_SUBSET_COLOR', False, validator=bool)
