@@ -7,7 +7,7 @@ import numpy as np
 import pandas as pd
 
 from glue.external import six
-from glue.core.message import (DataUpdateMessage,
+from glue.core.message import (DataUpdateMessage, DataRemoveComponentMessage,
                                DataAddComponentMessage, NumericalDataChangedMessage,
                                SubsetCreateMessage, ComponentsChangedMessage,
                                ComponentReplacedMessage)
@@ -63,7 +63,7 @@ class Data(object):
     See also: :ref:`data_tutorial`
     """
 
-    def __init__(self, label="", **kwargs):
+    def __init__(self, label="", coords=None, **kwargs):
         """
 
         :param label: label for data
@@ -72,7 +72,7 @@ class Data(object):
         Extra array-like keywords are extracted into components
         """
         # Coordinate conversion object
-        self.coords = Coordinates()
+        self.coords = coords or Coordinates()
         self._shape = ()
 
         # Components
@@ -177,6 +177,11 @@ class Data(object):
         """
         if component_id in self._components:
             self._components.pop(component_id)
+            if self.hub:
+                msg = DataRemoveComponentMessage(self, component_id)
+                self.hub.broadcast(msg)
+                msg = ComponentsChangedMessage(self)
+                self.hub.broadcast(msg)
 
     @contract(other='isinstance(Data)',
               cid='cid_like',
@@ -407,7 +412,7 @@ class Data(object):
             self._shape = component.shape
             self._create_pixel_and_world_components()
 
-        if self.hub and (not is_present):
+        if self.hub and not is_present:
             msg = DataAddComponentMessage(self, component_id)
             self.hub.broadcast(msg)
             msg = ComponentsChangedMessage(self)
@@ -698,19 +703,34 @@ class Data(object):
 
     @contract(old=ComponentID, new=ComponentID)
     def update_id(self, old, new):
-        """Reassign a component to a different :class:`glue.core.component_id.ComponentID`
+        """
+        Reassign a component to a different :class:`glue.core.component_id.ComponentID`
 
-        :param old: The old :class:`glue.core.component_id.ComponentID`.
-        :param new: The new :class:`glue.core.component_id.ComponentID`.
+        Parameters
+        ----------
+        old : :class:`glue.core.component_id.ComponentID`
+            The old component ID
+        new : :class:`glue.core.component_id.ComponentID`
+            The new component ID
         """
 
         if new is old:
             return
 
+        if new.parent is None:
+            new.parent = self
+
         changed = False
         if old in self._components:
-            self._components[new] = self._components[old]
+
+            # We want to keep the original order, so we can't just do:
+            #   self._components[new] = self._components[old]
+            # which will put the new component ID at the end, but instead
+            # we need to do:
+            self._components = OrderedDict((new, value) if key is old else (key, value)
+                                           for key, value in self._components.items())
             changed = True
+
         try:
             index = self._pixel_component_ids.index(old)
             self._pixel_component_ids[index] = new
@@ -730,7 +750,6 @@ class Data(object):
 
             # remove old component and broadcast the change
             # see #508 for discussion of this
-            self._components.pop(old)
             msg = ComponentReplacedMessage(self, old, new)
             self.hub.broadcast(msg)
 
