@@ -20,7 +20,6 @@ from __future__ import absolute_import, division, print_function
 import os
 
 from qtpy import QtGui, QtWidgets
-from glue.core.callback_property import CallbackProperty
 from glue.core import roi
 from glue.core.qt import roi as qt_roi
 from glue.utils.qt import get_qapp
@@ -197,6 +196,8 @@ class RoiMode(RoiModeBase):
     mouse release
     """
 
+    status_tip = "CLICK and DRAG to define selection"
+
     def __init__(self, viewer, **kwargs):
 
         super(RoiMode, self).__init__(viewer, **kwargs)
@@ -487,9 +488,6 @@ class ContrastMode(MouseMode):
         self._vmin = None
         self._vmax = None
 
-        if self._move_callback is None:
-            self._move_callback = self.viewer._set_norm
-
     def set_clip_percentile(self, lo, hi):
         """Percentiles at which to clip the data at black/white"""
         if lo == self._percent_lo and hi == self._percent_hi:
@@ -595,18 +593,10 @@ class ContrastMode(MouseMode):
         a.triggered.connect(nonpartial(setattr, self, 'stretch', 'log'))
         result.append(a)
 
-        a = QtWidgets.QAction("power", None)
-        a.triggered.connect(nonpartial(setattr, self, 'stretch', 'power'))
-        result.append(a)
-
-        a = QtWidgets.QAction("square root", None)
+        a = QtWidgets.QAction("sqrt", None)
         a.triggered.connect(nonpartial(setattr, self, 'stretch', 'sqrt'))
         result.append(a)
 
-        a = QtWidgets.QAction("squared", None)
-        a.triggered.connect(nonpartial(setattr, self, 'stretch', 'squared'))
-        result.append(a)
-
         a = QtWidgets.QAction("asinh", None)
         a.triggered.connect(nonpartial(setattr, self, 'stretch', 'arcsinh'))
         result.append(a)
